'-----------------------------------------------------------------------

'copyright                : (c) 2012, Werner Braun
'purpose                  : simple DDS control
'micro                    : Mega16
'suited for demo          : Yes
'commercial addon needed  : Yes, for your private use only
'version                  : 1.0

'------------------------------------------------------------------------

$regfile = "M16def.dat"       ' the used chip

$lib "double.lbx"
' We do not use software emulated I2C but the TWI.
$lib "i2c_twi.lbx"

$hwstack = 40       ' use 32 for the hardware stack
$swstack = 40       ' use 10 for the SW stack
$framesize = 40     ' use 40 for the frame space

'Config Serialin = Buffered , Size = 20
'Config Serialout = Buffered , SIZE = 20
'Enable Interrupts





' Master crystal
$crystal = 14745600

$baud = 38400       ' baud rate

'Pin    kind        (used for)

'PB0 :  XCK/T0      ()
'PB1 :  T1          ()
'PB2 :  INT2/AIN0   ()
'PB3 ;  OC0/AIN1    ()
'PB4 :  SS          ()
'PB5 :  MOSI        ()
'PB6 :  MISO        ()
'PB7 :  SCK         ()

'PD0 :  RXD         (UART serial in)
'PD1 :  TXD         (UART serial out)
'PD2 :  INT0        ()
'PD3 :  INT1        ()
'PD4 :  OC1B        ()
'PD5 :  OC1A        ()
'PD6 :  ICP1        ()
'PD7 :  OC2         ()

'PC0 .  SCL         (SCL TWI)
'PC1 :  SDA         (SDA TWI)
'PC2 :  TCK         ()
'PC3 :  TMS         ()
'PC4 :  TDO         ()
'PC5 :  TDI         ()
'PC6 :  TOSC1       ()
'PC7 :  TOSC2       ()

'PA0 :  ADC0        ()
'PA1 :  ADC1        ()
'PA2 :  ADC2        ()
'PA3 :  ADC3        ()
'PA4 :  ADC4        ()
'PA5 :  ADC5        ()
'PA6 :  ADC6        ()
'PA7 :  ADC7        ()

' Global variables

Dim Adder As Double
Dim dbl_Temp As Double
Dim Long_Adder As Long
Dim Str_temp As String * 20
Dim Str_UART As String * 20
Dim Command As Byte

' DDS variables

Dim DDSCrystal As Double
' TWI Slave (DDS) crystal
DDSCrystal = 16000000
Dim DDS_Adress As Byte
Dim Frequency As Double       ' Hz
Dim Modus As Byte
'
' r24,r25,r26,(r27) is the Adder value determining the frequency
Dim Reg24 As Byte
Dim Temp_r24 As Byte
Dim Reg25 As Byte
Dim Temp_r25 As Byte
Dim Reg26 As Byte
Dim Temp_r26 As Byte
Dim Reg27 As Byte
' r28,r29,r30,r31 is the max 32bit phase accumulator
Dim Reg28 As Byte
Dim Reg29 As Byte
Dim Reg30 As Byte
Dim Reg31 As Byte
Dim Temp_r31 As Byte
'
Const OffMode = &h00
Const NoiseMode = &H01
Const SawtoothMode_slow = &h02
Const SawtoothMode_fast = &h03
Const SineMode_slow = &h04
Const SineMode_fast = &h05
Const TriangleMode_slow = &h06
Const TriangleMode_fast = &h07
Const RectangularMode_raw = &h08
Const RectangularMode_fast = &h09
Const SawtoothMode_precise = &h0A
'
' Initialize TWI
' The chip will work in TWI/I2C master mode.

Config Scl = Portc.0       ' we need to provide the SCL pin name
Config Sda = Portc.1       ' we need to provide the SDA pin name
Config Twi = 100000 ' wanted clock frequency will set TWBR and TWSR
I2Cinit             ' we need to set the pins in the proper state

' Subs used

Declare Sub Send_TW(byval Register As Byte , Byval Value As Byte)
Declare Sub UpDateDDS()

Program:


DDS_Adress = 2
Rotate DDS_Adress , Left , 1       ' *2 ( LSL )   7 bit plus R/W
Frequency = 0       ' off at start
Adder = 0
Str_UART = ""


Reg31 = OffMode
call Send_TW(31 , 0)
Reg24 = 0
call Send_TW(24 , 0)
Reg25 = 0
call Send_TW(25 , 0)
Reg26 = 0
call Send_TW(26 , 0)
Reg28 = 0
call Send_TW(28 , 0)
Reg29 = 0
call Send_TW(29 , 0)
Reg30 = 0
call Send_TW(30 , 0)

Do
   ' scan serial port
   If Ischarwaiting() = 1 Then
      Inputbin Command
      Print Chr(command);
      If Command = 44 Then Command = 46       ' comma = dot
      Select Case Command
      Case 46       ' comma
         If Len(Str_UART) < 20 Then
             Str_UART = Str_UART + Chr(command)
         End If
      Case 48 To 57 ' number
         If Len(Str_UART) < 20 Then
             Str_UART = Str_UART + Chr(command)
         End If
      Case 97       ' A ~ OffMode
         Modus = 0
      Case 98       ' B ~ NoiseMode
         Modus = 1
      Case 99       ' C ~ SawtoothMode_slow
         Modus = 2
      Case 100      ' D ~ SawtoothMode_fast
         Modus = 3
      Case 101      ' E ~ SineMode_slow
         Modus = 4
      Case 102      ' F ~ SineMode_fast
         Modus = 5
      Case 103      ' G ~ TriangleMode_slow
         Modus = 6
      Case 104      ' H ~ TriangleMode_fast
         Modus = 7
      Case 105      ' I ~ RectangularMode_raw
         Modus = 8
      Case 106      ' J ~ RectangularMode_fast    > hardware
         Modus = 9
      Case 107      ' K ~ SawtoothMode_precise
         Modus = 10
      case 13
         print
         print str_UART
      case else
         Str_UART = ""
      End Select

      If Reg31 <> Modus Then
        ' mode has changed
        Str_UART = ""
        command = 0
        Call Send_TW(31 , Modus)
        Reg31 = Modus
        call UpDateDDS
      End If
      If command = 13 then
          if len(Str_UART) > 0 then
            Frequency = Val(Str_UART)
            call UpDAteDDS
         endif
         print Frequency
         Str_UART = ""
      End If
   End If

Loop

Sub UpDateDDS()



   Select Case Modus

   Case SawtoothMode_precise
      Adder = Frequency * 10
      Adder = Adder * 67108864

   Case SawtoothMode_slow
      Adder = Frequency * 8
      Adder = Adder * 33554432

   Case SawtoothMode_fast
      Adder = Frequency * 5
      Adder = Adder * 16777216

   Case SineMode_fast
      Adder = Frequency * 8
      Adder = Adder * 33554432

   Case SineMode_slow
      Adder = Frequency * 10
      Adder = Adder * 67108864

   Case TriangleMode_slow
      Adder = Frequency * 10
      Adder = Adder * 67108864

   Case TriangleMode_fast
      Adder = Frequency * 8
      Adder = Adder * 33554432

   Case RectangularMode_raw
      Adder = Frequency * 7
      Adder = Adder * 33554432

   Case RectangularMode_fast
      Adder = Frequency * 8
      Adder = Adder * 33554432

   End SELECT

   dbl_Temp = Adder / DDSCrystal
   Long_Adder = dbl_Temp

   Temp_r24 = Long_Adder Mod 256
   Rotate Long_Adder , Right , 8
   Temp_r25 = Long_Adder Mod 256
   Rotate Long_Adder , Right , 8
   Temp_r26 = Long_Adder Mod 256
   If Temp_r24 <> Reg24 Then
      Call Send_TW(24 , Temp_r24)
      Reg24 = Temp_r24
   End If
   If Temp_r25 <> Reg25 Then
      Call Send_TW(25 , Temp_r25)
      Reg25 = Temp_r25
   End If
   If Temp_r26 <> Reg26 Then
      Call Send_TW(26 , Temp_r26)
      Reg26 = Temp_r26
   End If

End Sub


Sub Send_TW(register As Byte , Value As Byte )
   'print Adder
   Print "SEND TWI ";
   Print Register;
   Print " ";
   Print Value
   I2Cstart
   I2Cwbyte DDS_Adress
   I2Cwbyte Register
   I2Cwbyte Value

End Sub

End